@@ -11,22 +11,24 @@ from logging import DEBUG, INFO, NOTSET, Handler, Logger
 from multiprocessing import Lock, Process, Value, current_process
 from time import time
 from typing import List, TYPE_CHECKING
+import warnings
 
+import attrs
 import numpy as np
-from attr import define, Factory
 
-from logprep.connector.connector_factory import ConnectorFactory
-from logprep.input.input import (
+from logprep._version import get_versions
+from logprep.abc.connector import Connector
+from logprep.abc.input import (
     CriticalInputError,
     FatalInputError,
     SourceDisconnectedError,
     WarningInputError,
 )
-from logprep.metrics.metric import Metric, MetricTargets, calculate_new_average
+from logprep.abc.output import CriticalOutputError, FatalOutputError, WarningOutputError
+from logprep.factory import Factory
+from logprep.metrics.metric import Metric, calculate_new_average, MetricTargets
 from logprep.metrics.metric_exposer import MetricExposer
-from logprep.output.output import CriticalOutputError, FatalOutputError, WarningOutputError
 from logprep.processor.base.exceptions import ProcessingWarning, ProcessingWarningCollection
-from logprep.processor.processor_factory import ProcessorFactory
 from logprep.util.multiprocessing_log_handler import MultiprocessingLogHandler
 from logprep.util.pipeline_profiler import PipelineProfiler
 from logprep.util.time_measurement import TimeMeasurement
@@ -57,13 +59,17 @@ class Pipeline:
     # pylint: disable=logging-not-lazy
     # Would require too much change in the tests.
 
-    @define(kw_only=True)
+    @attrs.define(kw_only=True)
     class PipelineMetrics(Metric):
         """Tracks statistics about a pipeline"""
 
         _prefix: str = "logprep_pipeline_"
 
-        pipeline: List["Processor.ProcessorMetrics"] = Factory(list)
+        input: Connector.ConnectorMetrics
+        """Input metrics"""
+        output: Connector.ConnectorMetrics
+        """Output metrics"""
+        pipeline: List["Processor.ProcessorMetrics"] = attrs.Factory(list)
         """Pipeline containing the metrics of all set processors"""
         kafka_offset: int = 0
         """The current offset of the kafka input reader"""
@@ -71,6 +77,7 @@ class Pipeline:
         """Mean processing time for one event"""
         _mean_processing_time_sample_counter: int = 0
 
+        # pylint: disable=not-an-iterable
         @property
         def number_of_processed_events(self):
             """Sum of all processed events of all processors"""
@@ -86,6 +93,8 @@ class Pipeline:
             """Sum of all errors of all processors"""
             return np.sum([processor.number_of_errors for processor in self.pipeline])
 
+        # pylint: enable=not-an-iterable
+
         def update_mean_processing_time_per_event(self, new_sample):
             """Updates the mean processing time per event"""
             new_avg, new_sample_counter = calculate_new_average(
@@ -99,10 +108,7 @@ class Pipeline:
     def __init__(
         self,
         pipeline_index: int,
-        connector_config: dict,
-        pipeline_config: List[dict],
-        metrics_config: dict,
-        timeout: float,
+        config: dict,
         counter: "SharedCounter",
         log_handler: Handler,
         lock: Lock,
@@ -111,9 +117,7 @@ class Pipeline:
     ):
         if not isinstance(log_handler, Handler):
             raise MustProvideALogHandlerError
-        self._connector_config = connector_config
-        self._pipeline_config = pipeline_config
-        self._timeout = timeout
+        self._logprep_config = config
         self._log_handler = log_handler
         self._logger = None
 
@@ -121,52 +125,80 @@ class Pipeline:
         self._pipeline = []
         self._input = None
         self._output = None
-
+        self._lock = lock
+        self._shared_dict = shared_dict
         self._processing_counter = counter
-
-        self._metrics_exposer = MetricExposer(metrics_config, metric_targets, shared_dict, lock)
+        self.metrics = None
+        self._metric_targets = metric_targets
+        self._metrics_exposer = None
         self._metric_labels = {"pipeline": f"pipeline-{pipeline_index}"}
-        self.metrics = self.PipelineMetrics(labels=self._metric_labels)
+
+        self._event_version_information = {
+            "logprep": get_versions().get("version"),
+            "configuration": self._logprep_config.get("version", "unset"),
+        }
 
     def _setup(self):
         self._create_logger()
-        self._build_pipeline()
         self._create_connectors()
+        self._create_metrics()
+        self._build_pipeline()
+
+    def _create_metrics(self):
+        self._metrics_exposer = MetricExposer(
+            self._logprep_config.get("metrics", {}),
+            self._metric_targets,
+            self._shared_dict,
+            self._lock,
+        )
+        self.metrics = self.PipelineMetrics(
+            input=self._input.metrics, output=self._output.metrics, labels=self._metric_labels
+        )
 
     def _build_pipeline(self):
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(f"Building '{current_process().name}'")
         self._pipeline = []
-        for entry in self._pipeline_config:
+        for entry in self._logprep_config.get("pipeline"):
             processor_name = list(entry.keys())[0]
             entry[processor_name]["metric_labels"] = self._metric_labels
-            processor = ProcessorFactory.create(entry, self._logger)
+            processor = Factory.create(entry, self._logger)
             self._pipeline.append(processor)
             self.metrics.pipeline.append(processor.metrics)
-            if self._logger.isEnabledFor(DEBUG):
+            if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
                 self._logger.debug(f"Created '{processor}' processor ({current_process().name})")
             self._pipeline[-1].setup()
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(f"Finished building pipeline ({current_process().name})")
 
     def _create_connectors(self):
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(f"Creating connectors ({current_process().name})")
-        self._input, self._output = ConnectorFactory.create(self._connector_config)
-        if self._logger.isEnabledFor(DEBUG):
+        input_connector_config = self._logprep_config.get("input")
+        connector_name = list(input_connector_config.keys())[0]
+        input_connector_config[connector_name]["metric_labels"] = self._metric_labels
+        input_connector_config[connector_name].update(
+            {"version_information": self._event_version_information}
+        )
+        self._input = Factory.create(input_connector_config, self._logger)
+        output_connector_config = self._logprep_config.get("output")
+        connector_name = list(output_connector_config.keys())[0]
+        output_connector_config[connector_name]["metric_labels"] = self._metric_labels
+        self._output = Factory.create(output_connector_config, self._logger)
+        self._output.input_connector = self._input
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(
-                f"Created input connector '{self._input.describe_endpoint()}' "
-                f"({current_process().name})"
+                f"Created input connector '{self._input.describe()}' " f"({current_process().name})"
             )
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(
-                f"Created output connector '{self._output.describe_endpoint()}' "
+                f"Created output connector '{self._output.describe()}' "
                 f"({current_process().name})"
             )
 
         self._input.setup()
         self._output.setup()
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug(f"Finished creating connectors ({current_process().name})")
 
     def _create_logger(self):
@@ -181,21 +213,24 @@ class Pipeline:
 
     def run(self):
         """Start processing processors in the Pipeline."""
-        self._setup()
+        with self._lock:
+            with warnings.catch_warnings():
+                warnings.simplefilter("default")
+                self._setup()
         self._enable_iteration()
         try:
-            if self._logger.isEnabledFor(DEBUG):
-                self._logger.debug("Start iterating (%s)", current_process().name)
+            if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
+                self._logger.debug(f"Start iterating ({current_process().name})")
             while self._iterate():
                 self._retrieve_and_process_data()
         except SourceDisconnectedError:
             self._logger.warning(
-                f"Lost or failed to establish connection to {self._input.describe_endpoint()}"
+                f"Lost or failed to establish connection to {self._input.describe()}"
             )
         except FatalInputError as error:
-            self._logger.error(f"Input {self._input.describe_endpoint()} failed: {error}")
+            self._logger.error(f"Input {self._input.describe()} failed: {error}")
         except FatalOutputError as error:
-            self._logger.error(f"Output {self._output.describe_endpoint()} failed: {error}")
+            self._logger.error(f"Output {self._output.describe()} failed: {error}")
 
         self._shut_down()
 
@@ -209,7 +244,11 @@ class Pipeline:
         event = {}
         try:
             self._metrics_exposer.expose(self.metrics)
-            event = self._input.get_next(self._timeout)
+            event, non_critical_error_msg = self._input.get_next(
+                self._logprep_config.get("timeout")
+            )
+            if non_critical_error_msg:
+                self._output.store_failed(non_critical_error_msg, event, None)
 
             try:
                 self.metrics.kafka_offset = self._input.current_offset
@@ -222,31 +261,28 @@ class Pipeline:
                 self._processing_counter.print_if_ready()
                 if event:
                     self._output.store(event)
-                    if self._logger.isEnabledFor(DEBUG):
+                    if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
                         self._logger.debug("Stored output")
         except SourceDisconnectedError as error:
             raise error
         except WarningInputError as error:
-            self._logger.warning(
-                f"An error occurred for input {self._input.describe_endpoint()}: {error}"
-            )
+            self._logger.warning(f"An error occurred for input {self._input.describe()}: {error}")
+            self._input.metrics.number_of_warnings += 1
         except WarningOutputError as error:
-            self._logger.warning(
-                f"An error occurred for output {self._output.describe_endpoint()}: {error}"
-            )
+            self._logger.warning(f"An error occurred for output {self._output.describe()}: {error}")
+            self._output.metrics.number_of_warnings += 1
         except CriticalInputError as error:
-            msg = f"A critical error occurred for input {self._input.describe_endpoint()}: {error}"
+            msg = f"A critical error occurred for input {self._input.describe()}: {error}"
             self._logger.error(msg)
             if error.raw_input:
                 self._output.store_failed(msg, error.raw_input, event)
+            self._input.metrics.number_of_errors += 1
         except CriticalOutputError as error:
-            msg = (
-                f"A critical error occurred for output "
-                f"{self._output.describe_endpoint()}: {error}"
-            )
+            msg = f"A critical error occurred for output " f"{self._output.describe()}: {error}"
             self._logger.error(msg)
             if error.raw_input:
                 self._output.store_failed(msg, error.raw_input, {})
+            self._output.metrics.number_of_errors += 1
 
     @TimeMeasurement.measure_time("pipeline")
     def _process_event(self, event: dict):
@@ -279,7 +315,7 @@ class Pipeline:
                         processor.metrics.number_of_warnings += 1
 
                 if not event:
-                    if self._logger.isEnabledFor(DEBUG):
+                    if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
                         self._logger.debug(f"Event deleted by processor {processor}")
                     return
         # pylint: disable=broad-except
@@ -299,7 +335,7 @@ class Pipeline:
         # pylint: enable=broad-except
 
     def _store_extra_data(self, extra_data: tuple):
-        if self._logger.isEnabledFor(DEBUG):
+        if self._logger.isEnabledFor(DEBUG):  # pragma: no cover
             self._logger.debug("Storing extra data")
         documents = extra_data[0]
         target = extra_data[1]
@@ -385,41 +421,33 @@ class MultiprocessingPipeline(Process, Pipeline):
     def __init__(
         self,
         pipeline_index: int,
-        connector_config: dict,
-        pipeline_config: List[dict],
-        metrics_config: dict,
-        timeout: float,
+        config: dict,
         log_handler: Handler,
-        print_processed_period: float,
         lock: Lock,
         shared_dict: dict,
-        profile: bool = False,
         metric_targets: MetricTargets = None,
     ):
         if not isinstance(log_handler, MultiprocessingLogHandler):
             raise MustProvideAnMPLogHandlerError
 
-        self._profile = profile
+        self._profile = config.get("profile_pipelines", False)
+        print_processed_period = config.get("print_processed_period", 300)
         self.processed_counter.setup(print_processed_period, log_handler)
 
         Pipeline.__init__(
             self,
-            pipeline_index,
-            connector_config,
-            pipeline_config,
-            metrics_config,
-            timeout,
-            self.processed_counter,
-            log_handler,
-            lock,
-            shared_dict,
+            pipeline_index=pipeline_index,
+            config=config,
+            counter=self.processed_counter,
+            log_handler=log_handler,
+            lock=lock,
+            shared_dict=shared_dict,
             metric_targets=metric_targets,
         )
 
         self._continue_iterating = Value(c_bool)
         with self._continue_iterating.get_lock():
             self._continue_iterating.value = False
-
         Process.__init__(self)
 
     def run(self):
