@@ -1,7 +1,6 @@
 # pylint: disable=missing-docstring
 # pylint: disable=protected-access
 # pylint: disable=line-too-long
-
 from copy import deepcopy
 from logging import getLogger
 
@@ -83,9 +82,18 @@ class TestConfiguration:
             "pipeline", [], '"pipeline" must contain at least one item!'
         )
 
-    def test_verify_verifies_connector_config(self):
+    def test_verify_verifies_input_config(self):
         self.assert_fails_when_replacing_key_with_value(
-            "connector", {"type": "unknown"}, 'Unknown connector type: "unknown"'
+            "input",
+            {"random_name": {"type": "unknown"}},
+            "Invalid connector configuration: Unknown type 'unknown'",
+        )
+
+    def test_verify_verifies_output_config(self):
+        self.assert_fails_when_replacing_key_with_value(
+            "output",
+            {"random_name": {"type": "unknown"}},
+            "Invalid connector configuration: Unknown type 'unknown'",
         )
 
     @pytest.mark.parametrize(
@@ -278,7 +286,7 @@ class TestConfiguration:
                 [
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: some_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: some_processor_name - Unknown type 'does_not_exist'",
                     )
                 ],
             ),
@@ -318,11 +326,11 @@ class TestConfiguration:
                 [
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: some_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: some_processor_name - Unknown type 'does_not_exist'",
                     ),
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: another_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: another_processor_name - Unknown type 'does_not_exist'",
                     ),
                 ],
             ),
@@ -369,7 +377,7 @@ class TestConfiguration:
                 [
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: some_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: some_processor_name - Unknown type 'does_not_exist'",
                     )
                 ],
             ),
@@ -409,11 +417,11 @@ class TestConfiguration:
                 [
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: some_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: some_processor_name - Unknown type 'does_not_exist'",
                     ),
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: another_processor_name - Unknown processor type 'does_not_exist'",
+                        "Invalid processor config: another_processor_name - Unknown type 'does_not_exist'",
                     ),
                 ],
             ),
@@ -427,10 +435,61 @@ class TestConfiguration:
                     ),
                     (
                         InvalidProcessorConfigurationError,
-                        "Invalid processor config: some_processor_name - The processor type specification is missing for processor with name 'some_processor_name'",
+                        "Invalid processor config: some_processor_name - The type specification is missing for element with name 'some_processor_name'",
                     ),
                 ],
             ),
+            (
+                "metrics configured without errors",
+                {
+                    "metrics": {
+                        "period": 10,
+                        "enabled": True,
+                        "cumulative": True,
+                        "aggregate_processes": True,
+                        "measure_time": {"enabled": True, "append_to_event": False},
+                        "targets": [
+                            {"prometheus": {"port": 8000}},
+                            {
+                                "file": {
+                                    "path": "./logs/status.json",
+                                    "rollover_interval": 86400,
+                                    "backup_count": 10,
+                                }
+                            },
+                        ],
+                    }
+                },
+                [],
+            ),
+            (
+                "measure_time enabled key is missing",
+                {
+                    "metrics": {
+                        "period": 10,
+                        "enabled": True,
+                        "cumulative": True,
+                        "aggregate_processes": True,
+                        "measure_time": {"append_to_event": False},
+                        "targets": [
+                            {"prometheus": {"port": 8000}},
+                            {
+                                "file": {
+                                    "path": "./logs/status.json",
+                                    "rollover_interval": 86400,
+                                    "backup_count": 10,
+                                }
+                            },
+                        ],
+                    }
+                },
+                [
+                    (
+                        RequiredConfigurationKeyMissingError,
+                        "Required option is missing: The following option keys for the measure time configs are missing: {'enabled'}",
+                    )
+                ],
+            ),
         ],
     )
     def test_verify_errors_get_collected(self, config_dict, raised_errors, test_case):
@@ -445,3 +504,19 @@ class TestConfiguration:
             assert errors_set == raised_errors, f"For test case '{test_case}'!"
         else:
             config._verify_metrics_config()
+
+    def test_verify_input_raises_missing_input_key(self):
+        config = deepcopy(self.config)
+        del config["input"]
+        with pytest.raises(
+            RequiredConfigurationKeyMissingError, match="Required option is missing: input"
+        ):
+            config._verify_input(logger)
+
+    def test_verify_output_raises_missing_output_key(self):
+        config = deepcopy(self.config)
+        del config["output"]
+        with pytest.raises(
+            RequiredConfigurationKeyMissingError, match="Required option is missing: output"
+        ):
+            config._verify_output(logger)
